/* 	Kurs programowania mikrokontrolerowe Megawin
		Projekt 07 - obsluga RTC */

#include "mg32f10x.h"
#include "hardware.h"
#include <stdlib.h>
#include <stdio.h>

volatile uint8_t pos = 0;
volatile uint8_t disp_time_flag = 0;

/* podwojnie buforowana obsluga zawartosci wyswietlacza */
volatile uint16_t disp_num = 0;
volatile uint16_t disp_num_shadow = 0;


char str[10];
uint8_t str_idx = 0;
uint32_t hh = 0, mm = 0, ss = 0;
			
int main(void)
{
	
	initPeripherals();
	
	UART_config(72000000, 9600);
	UART_print("Kurs EP - programowanie MCU Megawin!\n");
	UART_print("RTC test\n");
	
  /* wlaczenie taktowania domeny BACKUP */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_BKP, ENABLE);
  /* odblokowanie dostepu do domeny BACKUP */
	PWR_BackupAccessCmd(ENABLE);
  /* konfiguracja NVIC-a */
	NVIC_Config();
	
	/* sprawdzenie wartosci kontrolnej */
	/* jezeli rejestr DR1 = 0x1234 przechodzimy do normalnej pracy */
	/* jezeli nie, to inicjalizujemy RTC i rejestr DR1 */
	
	if (BKP_ReadBackupRegister(BKP_DR1) != 0x1234)
  {
		/* wstepna konfiguracja RTC */
		RTC_Config();
		/* zapisanie wartosci kontrolnej */
		BKP_WriteBackupRegister(BKP_DR1, 0x1234);
  }
  else
  {
    /* oczekiwanie na synchronizacje z szyna systemowa */
    RTC_WaitForSynchro();
    /* wlaczenie obslugi przerwania od sekund */
    RTC_ITConfig(RTC_IT_SEC, ENABLE);
    /* oczekiwanie na wykonanie ostatniej operacji */
    RTC_WaitForLastTask();
		/* ustawienie preskalera: 32,768 kHz/(32767+1) = 1 Hz */
    RTC_SetPrescaler(32767);
		/* oczekiwanie na wykonanie ostatniej operacji */
    RTC_WaitForLastTask();
  }
	
	
  while (1)
  {
		/* sprawdzenie flagi aktualizacji wyswietlacza (co 1 sekunde) */
		if(disp_time_flag){
			
			/* wlaczenie LED 1 */
			LED_onoff(1, LED_ON);
			/* zerowanie flagi */
			disp_time_flag = 0;
			
			/* reset licznika RTC po osiagnieciu pelnej doby (23:59:59) */
			if (RTC_GetCounter() >= 0x0001517F)
			{		
				 /* zerowanie licznika RTC */
				RTC_SetCounter(0x0);
				 /* oczekiwanie na wykonanie ostatniej operacji */
				 RTC_WaitForLastTask();
			}
			
			/* odczyt stanu licznika RTC */
			uint32_t timestmp = RTC_GetCounter();
			
			/* obliczenie godzin, minut i sekund */
			hh = timestmp / 3600;
			mm = (timestmp % 3600) / 60;
			ss = (timestmp % 3600) % 60;
			
			/* "konkatenacja" godzin i minut dla wyswietlacza LED */
			disp_num = hh*100 + mm;

			delay(10);
			/* wylaczenie diody LED 1 */
			LED_onoff(1, LED_OFF);

		}else{
			
			/* obsluga terminala jezeli nie trzeba aktualizowac wysw. LED */
			/* sprawdzenie czy odebrano znak z terminala */
			if(UART1->LSR & UART_LSR_DR){
				
				/* zapis znaku do stringa */
				str[str_idx] = UART1->RBR;
				/* inkrementacja indeksu stringa */
				str_idx++;
			
				if(str_idx > 7){
					
					str_idx = 0;
					
					/* pobranie wartosci z ciagu HH:MM:SS */
					sscanf(str, "%d:%d:%d", &hh, &mm, &ss);
					
					if((hh >=0) && (hh <=23)){
						if((mm >=0) && (mm <=59)){
							if((ss >=0) && (ss <=59)){
								
								/* przeliczenie nowej godziny na sekundy i aktualizacja licznika */
								RTC_SetCounter((uint32_t)hh*3600 + mm*60 + ss);
							}
						}
					}
				}
			}
		}
	}
}
