/* 	Kurs programowania mikrokontrolerowe Megawin
		Projekt 08 - obsluga interfejsu SPI */

#include "mg32f10x.h"
#include "hardware.h"

/* bufory do nadawania i odbioru */
char tx_buf[67];
char rx_buf[67];
/* indeksy (wskazniki pozycji w buforach) */
uint8_t tx_index = 0;
uint8_t rx_index = 0;
/* liczba bajtow do wyslania */
uint8_t tx_len = 0;
/* pomocniczy string do obslugi UART-a */
char str [67];

int main(void)
{

	initPeripherals();
	
	NVIC_Config();
	/* konfiguracja modulu SPI */
	SPI_Config();
	
	UART_config(72000000, 9600);
	
	UART_print("Kurs EP - programowanie MCU Megawin!\n");
	UART_print("SPI EEPROM\n\n");
	UART_print("Zapisany tekst: ");

	/* odczyt strony pamieci EEPROM (64 bajty) */
	
	/* komenda READ */
	tx_buf[0] = 0x03;
	/* 16-bitowy adres - poczatek pamieci*/
	tx_buf[1] = 0x00;
	tx_buf[2] = 0x00;
	
	/* 1 bajt komendy + 2 bajty adresu + 64 bajty pamieci */
	tx_len = 67;
	/* zerowanie indeksu bufora odbiorczego */
	rx_index = 0;
	
	/* wlaczenie przerwan */
	SPI_ITConfig(SPIM2, SPI_IT_RXF | SPI_IT_TXE, ENABLE);
	/* uruchomienie SPI */
	SPI_Cmd(SPIM2, ENABLE);

	/* oczekiwanie na zakonczenie odczytu */
	delay(5);

	/* dezaktywacja SPI */
	SPI_Cmd(SPIM2, DISABLE);

	/* zabezpieczenie przed wykroczeniem poza tablice */
	/* w czasie "drukowania" znakow funkcja UART_print() */
	rx_buf[66] = 0x00;
	
	/* kopia bufora z pominieciem pierwszych 3 znakow */
	for(uint8_t i = 3; i < 67; i++){
		str[i-3] = rx_buf[i];
	}
	/* wyslanie tekstu na UART */
	UART_print(str);
	
	UART_print("\nWpisz nowy tekst: >> ");

	/* odbieranie maks. 64 znakow */
	for(uint8_t i = 3; i < 67; i++){
		tx_buf[i] = UART_getchar();
		/* przerwanie petli jezeli wcisnieto enter */
		if(tx_buf[i] == '\n') break;
	}
	
	/* wyslanie komendy odblokowania zapisu EEPROM-u */
	tx_buf[0] = 0x06; // WREN
	/* wystarczy jeden bajt */
	tx_len = 1;
	/* wlaczenie przerwan i wyslanie komendy */ 
	SPI_ITConfig(SPIM2, SPI_IT_TXE, ENABLE);
  SPI_Cmd(SPIM2, ENABLE);
	delay(1);
  SPI_Cmd(SPIM2,DISABLE);

	/* komenda zapisu EEPROM-u */
	tx_buf[0] = 0x02;
	/* ten sam adres z ktorego byl odczyt */
	tx_buf[1] = 0x00;
	tx_buf[2] = 0x00;
	tx_len = 67;
	/* wlaczenie przerwan i wyslanie tekstu */
	SPI_ITConfig(SPIM2, SPI_IT_TXE, ENABLE);
  SPI_Cmd(SPIM2, ENABLE);

	/* na wszelki wypadek odczekujemy dluzej niz trzeba */
	delay(10);
  SPI_Cmd(SPIM2, DISABLE);

	UART_print("\nZapis zakonczony pomyslnie! \n");
	UART_print("Zapisany tekst: ");

	/* ponownie dokonujemy odczytu nowego tekstu z EEPROM-u */
	tx_buf[0] = 0x03;
	tx_buf[1] = 0x00;
	tx_buf[2] = 0x00;
	tx_len = 67;
	rx_index = 0;
	SPI_ITConfig(SPIM2, SPI_IT_RXF | SPI_IT_TXE, ENABLE);
	SPI_Cmd(SPIM2, ENABLE);
	delay(5);
  SPI_Cmd(SPIM2, DISABLE);
	
	/* wyslanie nowego tekstu na UART */
	rx_buf[66] = 0x00;
	
	for(uint8_t i = 3; i < 67; i++){
		str[i-3] = rx_buf[i];
	}
	UART_print(str);
	UART_print("\n");

		
  while (1)
  {
	
	}

}
